package com.tofly.filling.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.DateUtil;
import com.tofly.filling.entity.Filling;
import com.tofly.filling.entity.dto.FillingDto;
import com.tofly.filling.service.FillingService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 填报记录(Filling)表控制层
 *
 * @author HaiQiu
 * @since 2023-01-10 11:39:02
 */
@Api(tags = "填报记录")
@RestController
@RequestMapping("filling")
public class FillingController {
    /**
     * 服务对象
     */
    @Resource
    private FillingService fillingService;

    /**
     * 分页查询数据
     *
     * @param page    分页对象
     * @param filling 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "填报记录-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<Filling>> pageList(Page page, Filling filling,
                                                 @ApiParam(value = "查询时间，数组，开始时间-结束时间") @RequestParam(required = false) List<String> dates) {
        return ResultRespone.success(fillingService.pageList(page, filling, dates), "查询成功");
    }

    /**
     * 查询所有数据
     *
     * @param filling 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "填报记录-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<Filling>> allList(Filling filling) {
        return ResultRespone.success(fillingService.allList(filling), "查询成功");
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "填报记录-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<Filling> get(@PathVariable Serializable id) {
        return ResultRespone.success(fillingService.get(id), "查询成功");
    }

    /**
     * 新增数据
     *
     * @param fillingDto 实体
     * @return 新增结果
     */
    @ToFlyAppLog(title = "填报记录-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody FillingDto fillingDto) {
        fillingService.insert(fillingDto);
        return ResultRespone.success(null, "新增成功");
    }

    /**
     * 修改数据
     *
     * @param fillingDto 实体
     * @return 修改结果
     */
    @ToFlyAppLog(title = "填报记录-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@Validated @RequestBody FillingDto fillingDto) {
        fillingService.edit(fillingDto);
        return ResultRespone.success(null, "修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "填报记录-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        fillingService.delete(idList);
        return ResultRespone.success(null, "删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "填报记录-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        fillingService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "填报记录-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        fillingService.importData(file);
        return ResultRespone.success(null, "导入成功");
    }


    /**
     * 数据导出
     *
     * @param page    分页
     * @param filling 查询模型
     * @param isAll   是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "填报记录-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false否（默认否）"),
    })
    @GetMapping("/export")
    public void export(Page page, Filling filling, boolean isAll) {
        fillingService.export(page, filling, isAll);
    }


    public static void verificationDate(Date dateDb, Date nowDate) {
        if (!DateUtil.getDateFormat(dateDb, "yyyy-MM-dd")
                .equals(DateUtil.getDateFormat(nowDate, "yyyy-MM-dd"))) {
            throw new RuntimeException("数据仅限当天修改");
        }
    }
}

